
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;


public class Main {

    private static Connection conn = null;

    public static void main(String[] args) {
        if (!connectAndCreateTable()) {
            return;
        }

        deleteRecords();

        insertRecords();
        selectRecords();

        updateRecords();
        selectRecords();

        closeConnection();
    }


    private static boolean connectAndCreateTable() {
        try {
            // Create a databasae connection to the local database server
            conn = DriverManager.getConnection("jdbc:derby://localhost/testdb;create=true");

            // Check that our table exists
            DatabaseMetaData metaData = conn.getMetaData();

            if (!tablesArePresent(metaData)) {
            	createTables();
            }
        } catch (SQLException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            return false;
        }

        return true;
    }


	private static boolean tablesArePresent(DatabaseMetaData metaData) throws SQLException {
		// Filter only by table name (note the upper case), and type
		try (
			ResultSet tables = metaData.getTables(null, null, "INVENTORY", new String[] { "TABLE" });
		) {
		    // Check that the result set is empty
		    return tables.next();
		}
	}


	// Create a statement that will create the table
	private static void createTables() throws SQLException {
		try (Statement stmt = conn.createStatement()) {
		    stmt.execute(
		        "CREATE TABLE inventory (" +
		            "name VARCHAR(255)," +
		            "quantity INT," +
		            "price DECIMAL(6,2)" +
		        ")"
		    );
		}
	}

    private static void closeConnection() {
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException ex) {
                Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    private static void deleteRecords() {
        // Remove all items
        try (Statement stmt = conn.createStatement()) {
            int removedItems = stmt.executeUpdate("DELETE FROM inventory");
            
            if (removedItems > 0) {
                System.out.println(removedItems + " items removed.");
            }
        } catch (SQLException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static void insertRecords() {
        // Insert five items
        try (PreparedStatement stmt = conn.prepareStatement("INSERT INTO inventory VALUES(?, ?, ?)")) {
            for (int i = 0; i < 5; ++i) {
                stmt.clearParameters();
                stmt.setString(1, "Item #" + i);
                stmt.setInt(2, i + 2);
                stmt.setFloat(3, i * 1.41f);
                stmt.execute();
            }
        } catch (SQLException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static void selectRecords() {
        // List all records
        try (Statement stmt = conn.createStatement()) {
            ResultSet records = stmt.executeQuery("SELECT * FROM inventory");
            
            System.out.println();
            System.out.println("+---------------+-----------------------+---------------+");
            printColumnNamesLine(records);
            System.out.println("+---------------+-----------------------+---------------+");

            while (records.next()) {
                printRecordLine(records);
            }

            System.out.println("+---------------+-----------------------+---------------+");
            System.out.println();
        } catch (SQLException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }


	private static void printRecordLine(ResultSet records) throws SQLException {
		System.out.print("| " + records.getString(1) + "\t| ");
		System.out.print(records.getInt(2) + "\t\t\t| ");
		System.out.print(records.getFloat(3) + "\t\t|");
		System.out.println();
	}


	private static void printColumnNamesLine(ResultSet records) throws SQLException {
        System.out.print("| ");

        ResultSetMetaData metaData = records.getMetaData();
		for (int i = 1; i <= metaData.getColumnCount(); ++i) {
		    System.out.print(metaData.getColumnLabel(i) + "\t\t| ");
		}

		System.out.println();
	}

    private static void updateRecords() {
        System.out.println("Updating items: changing price 0 to 100...");

        try (PreparedStatement stmt = conn.prepareStatement("UPDATE inventory SET price = ? WHERE price = 0")) {
            stmt.clearParameters();
            stmt.setFloat(1, 100f);
            int updatedItems = stmt.executeUpdate();
            
            if (updatedItems > 0) {
                System.out.println(updatedItems + " items updated.");
            }
        } catch (SQLException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
