
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ServerMain {

    private static final int PORT = 1234;

    public static void main(String[] args) {
        try (
            // Open listener socket
            ServerSocket server = new ServerSocket(PORT);
        ) {
            // Indicate startup
            System.out.println("Server started to listen on port " + PORT);

            // Run the service continuously
            while(true) {
                // Look for a client connection, accept when found one
                Socket client = server.accept();

                // Process it on a separate thread
                ServerThread st = new ServerThread(client);
                st.start();
            }
        } catch (IOException ex) {
            Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static class ServerThread extends Thread {

        private Socket client;

        public ServerThread(Socket client) {
            this.client = client;
        }

        @Override
        public void run() {
            try {
                // Receive request: read out the method name
                InputStream is = client.getInputStream();
                ObjectInputStream ois = new ObjectInputStream(is);
                String methodName = ois.readUTF();

                // Read parameter values and types
                int paramCount = ois.readInt();

                // Read the value and type of every parameter
                Object[] params = new Object[paramCount];
                Class<?>[] types = new Class<?>[paramCount];

                for(int i = 0; i < paramCount; ++i) {
                    params[i] = ois.readObject();
                    types[i] = (Class<?>)ois.readObject();
                }

                // Initialize result
                InvocationResult result = null;

                // Create a service implementation instance
                CalculatorServiceImpl impl = new CalculatorServiceImpl();

                // Invoke the service with reflection
                Class<? extends CalculatorServiceImpl> k = impl.getClass();
                try {
                    Method method = k.getDeclaredMethod(methodName, types);

                    // Setting the result field in the result wrapper
                    result = new InvocationResult(method.invoke(impl, params));
                } catch (NoSuchMethodException ex) {
                    // In this case, the client wanted to use an operation
                    // which is not defined by the service interface.
                    Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
                } catch (InvocationTargetException ex) {
                    // Setting the exception field in the result wrapper
                    result = new InvocationResult(ex.getCause());
                } catch (SecurityException | IllegalAccessException | IllegalArgumentException ex) {
                    Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
                }

                // Create the output stream
                OutputStream os = client.getOutputStream();
                ObjectOutputStream oos = new ObjectOutputStream(os);

                // Send response: write in the result
                oos.writeObject(result);
                oos.flush(); // important: immediately write contents to the socket
            } catch (IOException | ClassNotFoundException ex) {
                Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
            } finally {
                // Close connection to the client
                try {
                    client.close();
                } catch (IOException ex) {
                    Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }
}
