package example;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

// Demonstrates asynchronous message receiving through a JMS queue.
// It uses the common JMS API, which is available from version 1.1.
public class Main {

	public static void main(String[] args) throws NamingException, JMSException {
		// Lookup resources from the application server.
		Context ctx = new InitialContext();
		// Use the default JMS connection factory of GlassFish 
		ConnectionFactory connectionFactory = (ConnectionFactory)ctx.lookup(
			"java:comp/DefaultJMSConnectionFactory"
		);
		// The message destination is a custom message queue. This needs to be
		// set up in the administration console before running this application.
		Destination destination = (Destination)ctx.lookup(
			"jms/ExampleQueue"
		);
		ctx.close();
		
		withConnection(connectionFactory, destination, 
			consumer -> {
				try {
					consumer.setMessageListener(m -> {
						// This method is called in a separate thread every time the
						// application asynchronously receives a single message.
						// The web application sends only text messages to the
						// queue, and all of them have a `SenderUserAgent` property.
						TextMessage message = (TextMessage)m;
						try {
							System.out.printf("Message received: `%s` (user agent: %s)%n",
									message.getText(),
									message.getStringProperty("SenderUserAgent"));
						} catch (JMSException e) {
							e.printStackTrace();
						}
					});
				} catch (JMSException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			});
	}

	private static void withConnection(ConnectionFactory connectionFactory, Destination destination, java.util.function.Consumer<MessageConsumer> cons)
			throws JMSException {
		// Create a connection
		try (Connection connection = connectionFactory.createConnection()) {
			// And an associated session
			Session session = connection.createSession();
			// With a message consumer
			MessageConsumer consumer = session.createConsumer(destination);

//			// Add an asynchronous message listener
//			consumer.setMessageListener(new MessageListener() {
//				
//				// This method is called in a separate thread every time the
//				// application asynchronously receives a single message.
//				@Override
//				public void onMessage(Message m) {
//					// The web application sends only text messages to the
//					// queue, and all of them have a `SenderUserAgent` property.
//					TextMessage message = (TextMessage)m;
//					try {
//						System.out.println("Message received: `"
//							+ message.getText()
//							+ "` (user agent: "
//							+ message.getStringProperty("SenderUserAgent")
//							+ ")");
//					} catch (JMSException e) {
//						e.printStackTrace();
//					}
//				}
//			});

			cons.accept(consumer);

			// Start delivery of incoming messages
			connection.start();
			
			System.out.println("Receiving messages asynchronously ...");

			while(true) {
				try { Thread.sleep(2000); } catch (InterruptedException e) {}
			}
		}
	}

}
