package example;

import java.io.IOException;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// A simple servlet which can insert messages into a JMS queue.
// It uses the common JMS API, which is available from version 1.1.

// To run the examples, a new message queue named
// `jms/ExampleQueue` needs to be created in the GlassFish admin console.
@WebServlet("/")
public class MessageGenerator extends HttpServlet {
	private static final long serialVersionUID = 1L;
    
	// Use the default JMS connection factory of GlassFish.
	// From JMS 2.0 it is required to provide a default factory named like this.
	@Resource(lookup="java:comp/DefaultJMSConnectionFactory")
	private ConnectionFactory connectionFactory;
	
	// The message destination is a custom message queue. This needs to be
	// set up in the administration console before running this application.
	// To make it more flexible use the Destination interface instead of Queue.
	@Resource(lookup="jms/ExampleQueue")
	private Destination destination;
	
	private Connection connection;
	
	@PostConstruct
	private void initializeJMSConnection() {
		// Create a connection which can be used from multiple threads
		try {
			connection = connectionFactory.createConnection();
		} catch (JMSException e) {
			throw new RuntimeException(e);
		}
	}
	
	@PreDestroy
	private void closeJMSConnection() {
		try {
			connection.close();
		} catch (JMSException e) {
			throw new RuntimeException(e);
		}
	}
	
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// Redirect the request to a JSP page
		request.getRequestDispatcher("/WEB-INF/form.jsp").forward(request, response);
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// Fetch user input
		String value = request.getParameter("value");
		
		// Do not add empty messages to the queue
		if (value != null && value.trim().length() > 0) {
			// Create a session which is a single-threaded object
			try (Session session = connection.createSession() ) {
				sendMsg(session, makeMsg(request, value, session));
			} catch (JMSException e) {
				// Log the message and send and error page
				e.printStackTrace();
				response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e.getMessage());
				return;
			}
		}
		
		// Redirect to the original page
		response.sendRedirect(request.getRequestURI());
	}

	private void sendMsg(Session session, TextMessage message)
			throws JMSException {
		// With a message producer assigned to the queue
		MessageProducer producer = session.createProducer(destination);
		// And send it to the queue through the message producer
		producer.send(message);
	}

	private TextMessage makeMsg(HttpServletRequest request, String value,
			Session session) throws JMSException {
		// Create and assemble a new text message
		TextMessage message = session.createTextMessage();
		// Fill the message body
		message.setText(value);
		// A property is a custom, optional message header
		message.setStringProperty("SenderUserAgent", request.getHeader("User-Agent"));
		return message;
	}

}
