package example;

import java.io.IOException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebInitParam;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;


@WebServlet(
	name = "gameServlet",
	description = "Implements a simple number guessing game.",
	urlPatterns = { "/" },
	initParams = { @WebInitParam(name = "showSolution", value = "false") })
public class GameServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	
	// Stores the "showSolution" initialization parameter
	private boolean showSolution;
	
	@Override
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		
		// Read initialization parameter
		showSolution = Boolean.parseBoolean(config.getInitParameter("showSolution"));
	}

	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// Ensure that a GameBean instance is set in the session scope,
		// no matter what HTTP method is used
		HttpSession session = request.getSession();
		GameBean gb = (GameBean)session.getAttribute("gameBean");
		
		if (null == gb) {
			gb = new GameBean();
			session.setAttribute("gameBean", gb);
		}
		
		super.service(request, response);
	}

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		HttpSession session = request.getSession();
		GameBean gb = (GameBean)session.getAttribute("gameBean");
		
		// Note that we call equals on a fixed string object,
		// not on the request parameter - which can be a null reference.
		if (gb.isFoundTheSolution() && "true".equals(request.getParameter("reset"))) {
			// Reset the bean if the solution is found
			// (starts a new game by generating a new solution)
			gb.reset();
		}
		
		// Pass the initialization parameter to the view as a "request scope variable"
		request.setAttribute("showSolution", showSolution);
		
		// Render the form with messages
		request.getRequestDispatcher("/WEB-INF/guess.jsp").forward(request, response);
		
		// Clear indicator flags (use them like a "flash scope")
		gb.clearFlags();
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// Retrieve the instance of GameBean from the session scope
		HttpSession session = request.getSession();
		GameBean gb = (GameBean)session.getAttribute("gameBean");
		
		// Check the guess and set some internal state
		if (gb.checkSolution(request.getParameter("guess"))) {
			// Set the "high scores" in application scope
			// This needs mutual exclusion on the usage of the servlet context
			synchronized(this) {
				Integer minGuesses = (Integer)getServletContext().getAttribute("minGuesses");
			
				if (null == minGuesses || gb.getNumberOfGuesses() < minGuesses) {
					getServletContext().setAttribute("minGuesses", gb.getNumberOfGuesses());
				}
			}
		}
		
		// Use the post-redirect-get design pattern,
		// by sending an external redirect to the browser
		// (it will get the same page with another method)
		response.sendRedirect(response.encodeRedirectURL(request.getRequestURI()));
	}
}
