package example;

import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.ejb.ConcurrencyManagement;
import javax.ejb.ConcurrencyManagementType;
import javax.ejb.Local;
import javax.ejb.Lock;
import javax.ejb.LockType;
import javax.ejb.Remote;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.interceptor.Interceptors;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

// Defines a singleton session bean. Each running instance of JVM will have
// exactly one instance of this class.
@Singleton
// Ensure the instantiation will be done right after application deployment.
@Startup
// This service has different local and remote interfaces.
@Local(ResultPersistence.class)
@Remote(ResultStorage.class)
@Interceptors({ Logger.class })
// Setup the concurrency management. `CONTAINER` is the default value,
// so this annotation is redundant here.
@ConcurrencyManagement(ConcurrencyManagementType.CONTAINER)
public class ResultStorageBean implements ResultStorage, ResultPersistence {

	// Inject a persistence context. Because there is only one persistence unit
	// is defined in the presistence.xml, the unitName property can be omitted.
	@PersistenceContext
	private EntityManager em;
	
	// An internal result cache, which will be maintained in the same way as the
	// database storage, so they will contain the same entities.
	private List<Result> resultCache;
	
	// After the dependency injection is done, and `em` is set up to an entity
	// manager reference, we can query the database to fill the in-memory cache
	// with existing result entries.
	@PostConstruct
    public void loadExistingData() {
        resultCache = em.createQuery("SELECT r FROM Result r", Result.class).getResultList();
        
        System.out.println("Items loaded to cache: " + resultCache.size());
    }

	// By default, when `ConcurrencyManagementType` is `CONTAINER`, all business
	// methods have an implicit @Lock(WRITE), which means that only one thread
	// can execute the method body at the same time.
	// The lock holds the singleton instance itself, so all calls of any write
	// methods are mutually exclusive. Note that this can cause a huge
	// performance bottleneck.
	@Override
	public void persistWithTimestamp(int result) {
		Result r = new Result(new Date(), result);
		em.persist(r);
		resultCache.add(r);
	}
	
	// Clients can execute the body of this method simultaneously.
	@Lock(LockType.READ)
    public List<Result> getAllResults() {
		return resultCache;
    }

}
